

/*
   Smart Thermostat by Roque Campos
   Sketch to read temperatures with an ESP32 board and a DHTxx sensor
   Complete Project Details https://www.smart-thermostat.eu
*/
#include <WiFi.h>
#include <DHTesp.h>
#include "esp_system.h"
#include "soc/rtc.h"
#include "rom/uart.h"

/****** Start configuration section ******/


const char* ssid     = "YourWifiName"; //Replace with your network SSID
const char* password = "YourPassword"; //Replace with your network password
const String nodeName = "NodeName"; //Replace with the name for this node (example "Beed Room")




//Uncomment the appropiate type for your DHT Sensor.
//DHTesp::DHT_MODEL_t DHT_TYPE = DHTesp::DHT22;
DHTesp::DHT_MODEL_t DHT_TYPE = DHTesp::DHT11;

/****** End configuration section ******/
/****** Please don't touch anything below this point unless you know what you are doing. ******/

//Don't touch anything below this point unless you know what you are doing.
const String sketchVersion = "3";
const String nodeType = "TH";
String thError = "";

WiFiServer server(80);

const int ONBOARD_LED_PIN = 2;
const int DHT_PIN = 26;

DHTesp dht;

void setup() {
  
  Serial.begin(115200);
  slowDownCpu();
  pinMode(ONBOARD_LED_PIN, OUTPUT);

  dht.setup(DHT_PIN, DHT_TYPE);
  setUpWifi();
}

void slowDownCpu() {
  rtc_clk_cpu_freq_set(RTC_CPU_FREQ_160M);
  uart_tx_wait_idle(0);
  int clockspeed = rtc_clk_cpu_freq_get();

  char* clockSpeeds[5] = {"XTAL", "80Mhz", "160Mhz", "240Mhz", "2Mhz"};
  Serial.print("Setting CPU freq to: ");
  Serial.println(clockSpeeds[clockspeed]);
  Serial.println("");
}
void setUpWifi() {

  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);


  // attempt to connect to Wifi network:
  int counter = 0;
  while (WiFi.status() != WL_CONNECTED) {
    Serial.println("Disconnecting");
    WiFi.disconnect();
    Serial.println("Connecting");
    WiFi.begin(ssid, password);
    int counter = 0;
    while (WiFi.status() != WL_CONNECTED) {
      Serial.println("Waiting for connection " + String(counter));
      delay(1000);
      counter++;
      if (counter > 10) {
        Serial.println("Reached " + String(counter) + " attempts, breaking");
        break;
      }
    }
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.print("MAC: ");
  Serial.println(WiFi.macAddress());
  // print the received signal strength:
  long rssi = WiFi.RSSI();
  Serial.print("Signal strength (RSSI):");
  Serial.print(rssi);
  Serial.println(" dBm");
  server.begin();

}
double celsius2Fahrenheit(double celsius)
{
  return celsius * 9 / 5 + 32;
}

void loop() {


  if (WiFi.status() != WL_CONNECTED) {
    Serial.println("Disconnected wifi detected. Setting up again");
    setUpWifi();
  } else {
    // listen for incoming clients
    WiFiClient client = server.available();
    if (client.connected()) {
      digitalWrite(ONBOARD_LED_PIN, HIGH);
      String req = client.readStringUntil('\r');
      Serial.println("Received request: " + req);
      //client.flush();

      double humidity = dht.getHumidity();
      double celsiusTemp = dht.getTemperature();
      double fahrenheitTemp = -10000;

      if (dht.getStatus() != 0) {
        thError = String(dht.getStatusString());
        Serial.println("DHT Error status: " + thError);
        humidity = -10000;
        celsiusTemp = -10000;
      } else {
        thError = "";
        Serial.println("DHT OK");
        fahrenheitTemp = celsius2Fahrenheit(celsiusTemp);
      }
      // send a standard http response header
      String response = String();
      response += "HTTP/1.1 200 OK\n";
      response += "Content-Type: application/json\n";
      response += "\n"; //This carry return is very important. Without it the response will not be sent.
      response += "{\n";
      response += "\"type\":\"" + nodeType + "\",\n";
      response += "\"name\":\"" + nodeName + "\",\n";
      response += "\"version\":\"" + sketchVersion + "\",\n";
      response += "\"th_error\":\"" + thError + "\",\n";
      response += "\"mac\":\"" + WiFi.macAddress() + "\",\n";

      String cc = String(celsiusTemp);
      cc.trim();
      response += "\"celsius\":\"" + cc + "\",\n";

      String ff = String(fahrenheitTemp);
      ff.trim();
      response += "\"fahrenheit\":\"" + ff + "\",\n";

      String hh = String(humidity);
      hh.trim();
      response += "\"humidity\":\"" + hh + "\",\n";

      long rssi = WiFi.RSSI();
      response += "\"wifi_ssi\": \"" + String(rssi) + "\"\n";
      response += "}\n";
      response += "\n";
      Serial.println(response);
      client.println(response);
      delay(100);
      client.stop();
      digitalWrite(ONBOARD_LED_PIN, LOW);
      Serial.println("_________________________________________");
    }

  }
}
